<?php

namespace Teknisa\ACL\Repositories;

use Teknisa\ACL\Util\InstanceProvider;
use Teknisa\ACL\Repositories\BaseCrud;
class Operator extends BaseCrud {

    protected $table = 'OPERADOR';

    public function __construct()
    {
        parent::__construct($this->table);
    }

    public function findOperators($organizationId, $filters, $limit, $offset, $filterParcNegocio = '', $oAccCustomOrgId = null, $orgOperFilter = '')
    {
        $nrOrgOperQuery = $oAccCustomOrgId ? "'$oAccCustomOrgId'" : "OPER.NRORG";
        $query = <<<SQL
            SELECT OPER.*
              FROM (SELECT DISTINCT OPER.CDOPERADOR, PARC.NRINSCRICAOPARC, PARC.NMPRINCIPALPARC AS NMPARCNEGOCIO, OPER.NMACESSO, 
                           OPER.CDPAIS, OPER.NMOPERADOR, OPER.DTULTATUOPER, OPER.CDCLIENTE, OPER.CDEMPRESA, OPER.DSEMAILOPER, OPER.IDOPERMASTER,
                           OPER.IDOPERACESSO, OPER.IDOPERATIVO, OPER.NRORG, OPER.DTINCLUSAO, OPER.DTULTATU,
                           OPER.NRORGINCLUSAO, OPER.CDOPERINCLUSAO, OPER.CDOPERULTATU, OPER.NRORGULTATU,
                           OPER.NRPARCNEGOCIO, OPER.NRORGSELECIONADA, OPER.CDFILIALSELECIONADA,
                           OPER.NRORGTRAB, OPER.NRESTRUTTRAB, OPER.IDATIVO, OPER.IDTPOPERADOR, ORG.NMORGANIZACAO, EST.NMESTRUTURAM, 
                           CLI.NMRAZSOCCLIE, PA.NMPAIS, OPER.DTULTACESOPER, V.CDVENDEDOR, V.NMRAZSOCVEN, V.CDFILIAL, V.CDCAIXA, F.NMFILIAL, C.NMCAIXA,
                           (CASE NVL(V.CDVENDEDOR,'NULL') WHEN 'NULL' THEN 0 ELSE 1 END ) AS ADD_SELLER
                      FROM OPERADOR OPER
                 LEFT JOIN ORGANIZACAO ORG ON ORG.NRORG = OPER.NRORG
                 LEFT JOIN ESTRUTURAM EST ON EST.NRORG = OPER.NRORG 
                                         AND EST.NRESTRUTURAM = OPER.NRESTRUTTRAB
                 LEFT JOIN CLIENTE CLI ON CLI.NRORG = OPER.NRORG
                                      AND CLI.CDCLIENTE = OPER.CDCLIENTE
                 LEFT JOIN PAIS PA ON PA.CDPAIS = OPER.CDPAIS
                 LEFT JOIN PARCNEGOCIO PARC ON PARC.NRORG = OPER.NRORG
                                           AND PARC.NRPARCNEGOCIO = OPER.NRPARCNEGOCIO
                 LEFT JOIN VENDEDOR V ON OPER.CDOPERADOR = V.CDOPERADOR AND V.NRORG  = OPER.NRORG
                 LEFT JOIN FILIAL F ON V.CDFILIAL = F.CDFILIAL AND V.NRORG  = F.NRORG
                 LEFT JOIN CAIXA C ON V.CDCAIXA = C.CDCAIXA AND V.NRORG  = C.NRORG AND C.CDFILIAL = F.CDFILIAL
                     WHERE OPER.CDOPERADOR <> '#SUP_TEKNISA'
                     $orgOperFilter) OPER
             WHERE OPER.NRORG = $organizationId
             $filterParcNegocio
             $filters
          ORDER BY OPER.NMOPERADOR ASC
SQL;

        $queryWithPagination = $this->getPaginatedQuery($query, $limit, $offset);
        return InstanceProvider::getEntityManager()->getConnection()->fetchAll($queryWithPagination);
    }

    public function findOperatorOrgOper($cdoperador, $organizationId)
    {
        $query = <<<SQL
            SELECT ORL.NRORGOPER
              FROM ORGRELACOPER ORL
             WHERE ORL.CDOPERADOR = :CDOPERADOR
               AND ORL.NRORG = :NRORG
SQL;

        $params = array (
            'CDOPERADOR' => $cdoperador,
            'NRORG'      => $organizationId,
        );

        return InstanceProvider::getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findAccessHour($cdoperador, $organizationId)
    {
        $query = <<<SQL
            SELECT HR.ID, HR.HORACESSOINI, HR.HORACESSOFIM, 'N' AS OPERSEMRESTRICAO,
                   RTRIM(
                            XMLAGG(
                                XMLELEMENT(E, HD.DIASEMANA, ',').EXTRACT('//text()') ORDER BY HD.DIASEMANA
                            ).GetClobVal(), ','
                        ) AS DIASEMANA
              FROM HORACESSOPER HR
              JOIN HORACESSOPERDIA HD ON HD.HORACESSOPER_ID = HR.ID
             WHERE HR.CDOPERADOR = :CDOPERADOR
               AND HR.NRORG = :NRORG
          GROUP BY HR.ID, HR.HORACESSOINI, HR.HORACESSOFIM
SQL;

        $params = array (
            'CDOPERADOR' => $cdoperador,
            'NRORG'      => $organizationId,
        );

        return InstanceProvider::getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function insertAccessHour($cdoperador, $hrIni, $hrFim, $id, $organizationId)
    {
        $query = <<<SQL
            INSERT INTO HORACESSOPER (ID, HORACESSOINI, HORACESSOFIM, CDOPERADOR, NRORG, DTINCLUSAO)
                 VALUES (:ID, :HORACESSOINI, :HORACESSOFIM, :CDOPERADOR, :NRORG, SYSDATE)
SQL;

        $params = array (
            'ID'           => $id,
            'HORACESSOINI' => $hrIni,
            'HORACESSOFIM' => $hrFim,
            'CDOPERADOR'   => $cdoperador,
            'NRORG'        => $organizationId,
        );

        InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
    }

    public function updateAccessHour($cdoperador, $hrIni, $hrFim, $id, $organizationId)
    {
        $query = <<<SQL
            UPDATE HORACESSOPER
               SET HORACESSOINI = :HORACESSOINI, 
                   HORACESSOFIM = :HORACESSOFIM,
                   DTULTATU     = SYSDATE
             WHERE CDOPERADOR = :CDOPERADOR
               AND NRORG = :NRORG
               AND ID = :ID
SQL;

        $params = array (
            'ID'           => $id,
            'HORACESSOINI' => $hrIni,
            'HORACESSOFIM' => $hrFim,
            'CDOPERADOR'   => $cdoperador,
            'NRORG'        => $organizationId,
        );

        InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
    }

    public function deleteAccessHour($cdoperador, $organizationId)
    {
        $query = <<<SQL
            DELETE
              FROM HORACESSOPER
             WHERE CDOPERADOR = :CDOPERADOR
               AND NRORG = :NRORG
SQL;

        $params = array (
            'CDOPERADOR' => $cdoperador,
            'NRORG'      => $organizationId,
        );

        InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
    }

    public function insertAccessHourDay($accessHourId, $id, $weekDay, $organizationId)
    {
        $query = <<<SQL
            INSERT INTO HORACESSOPERDIA (ID, HORACESSOPER_ID, DIASEMANA, NRORG, DTINCLUSAO)
                 VALUES (:ID, :HORACESSOPER_ID, :DIASEMANA, :NRORG, SYSDATE)
SQL;

        $params = array (
            'ID'              => $id,
            'HORACESSOPER_ID' => $accessHourId,
            'DIASEMANA'       => $weekDay,
            'NRORG'           => $organizationId,
        );

        InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
    }

    public function deleteAccessHourDay($accessHourId, $organizationId)
    {
        $query = <<<SQL
            DELETE
              FROM HORACESSOPERDIA
             WHERE HORACESSOPER_ID = :HORACESSOPER_ID
               AND NRORG = :NRORG
SQL;

        $params = array (
            'HORACESSOPER_ID' => $accessHourId,
            'NRORG'           => $organizationId
        );

        InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
    }

    public function findOperatorsBranchProfile($organizationId, $filters, $limit, $offset, $filterParcNegocio = '', $joinPerfAcessControl, $nrAccessProfile, $modeParametrization, $oAccCustomOrgId = null, $orgOperFilter = '')
    {
        $nrOrgOperQuery = $oAccCustomOrgId ? "'$oAccCustomOrgId'" : "OPER.NRORG";
        $query = <<<SQL
            SELECT DISTINCT OPER.*                         
              FROM (SELECT OPER.CDOPERADOR,  OPER.NMACESSO,  PARC.NRINSCRICAOPARC, OPER.CDPERFILSELECIONADO,
                           OPER.CDPAIS, OPER.NMOPERADOR, OPER.DTULTATUOPER, OPER.CDCLIENTE, OPER.CDEMPRESA, OPER.DSEMAILOPER, OPER.IDOPERMASTER,
                           OPER.IDOPERACESSO, OPER.IDOPERATIVO, OPER.NRORG, OPER.DTINCLUSAO, OPER.DTULTATU,
                           OPER.NRORGINCLUSAO, OPER.CDOPERINCLUSAO, OPER.CDOPERULTATU, OPER.NRORGULTATU,
                           OPER.NRPARCNEGOCIO, OPER.NRORGSELECIONADA, OPER.CDFILIALSELECIONADA, ORG.NRINSJUREMPR,
                           OPER.NRORGTRAB, OPER.NRESTRUTTRAB, OPER.IDATIVO, OPER.IDTPOPERADOR, ORG.NMORGANIZACAO, EST.NMESTRUTURAM, 
                           CLI.NMRAZSOCCLIE, PA.NMPAIS, OPER.DTULTACESOPER, V.CDVENDEDOR, V.NMRAZSOCVEN, V.CDFILIAL, V.CDCAIXA, F.NMFILIAL, C.NMCAIXA,
                           (CASE NVL(V.CDVENDEDOR,'NULL') WHEN 'NULL' THEN 0 ELSE 1 END ) AS ADD_SELLER
                      FROM OPERADOR OPER
                 LEFT JOIN ORGANIZACAO ORG ON ORG.NRORG = OPER.NRORG
                 LEFT JOIN ESTRUTURAM EST ON EST.NRORG = OPER.NRORG 
                                         AND EST.NRESTRUTURAM = OPER.NRESTRUTTRAB
                 LEFT JOIN CLIENTE CLI ON CLI.NRORG = OPER.NRORG
                                      AND CLI.CDCLIENTE = OPER.CDCLIENTE
                 LEFT JOIN PARCNEGOCIO PARC ON PARC.NRORG = OPER.NRORG
                                           AND PARC.NRPARCNEGOCIO = OPER.NRPARCNEGOCIO
                 LEFT JOIN PAIS PA ON PA.CDPAIS = OPER.CDPAIS
                 LEFT JOIN VENDEDOR V ON OPER.CDOPERADOR = V.CDOPERADOR AND V.NRORG  = OPER.NRORG
                 LEFT JOIN FILIAL F ON V.CDFILIAL = F.CDFILIAL AND V.NRORG  = F.NRORG
                 LEFT JOIN CAIXA C ON V.CDCAIXA = C.CDCAIXA AND V.NRORG  = C.NRORG AND C.CDFILIAL = F.CDFILIAL
                $joinPerfAcessControl
                     WHERE OPER.CDOPERADOR <> '#SUP_TEKNISA'
                     $orgOperFilter) OPER
             WHERE OPER.NRORG = $organizationId
          $filters
          ORDER BY OPER.NMOPERADOR ASC
SQL;
        $queryWithPagination = $this->getPaginatedQuery($query, $limit, $offset);
        if(!$modeParametrization && ($nrAccessProfile == null)){
            return array();
        } else {
            return InstanceProvider::getEntityManager()->getConnection()->fetchAll($queryWithPagination);
        }
    }

    private function getPaginatedQuery($query, $limit, $offset)
    {
        if ($limit > 0) {
            $max = $limit;
            $column = '*';
            if ($offset > 0) {
                $min = $offset + 1;
                $query = 'SELECT * FROM (SELECT a.' . $column . ', rownum AS doctrine_rownum FROM (' .
                    $query .
                    ') a WHERE rownum <= ' . $max . ') WHERE doctrine_rownum >= ' . $min;
            } else {
                $query = 'SELECT a.' . $column . ' FROM (' . $query . ') a WHERE ROWNUM <= ' . $max;
            }
        }
        return $query;
    }

    public function saveOperator(
            $operatorCode, $activeOperatorId, $activeId, $operatorName, $accessName, $operatorEmail,
            $countryCode, $businessPartnership, $clientCode, $organizationId, $accessOperatorId,
            $selectedProfile, $organizationInclusion, $operatorInclusionCode, $password, $idopermaster, $nrestruttrab,
            $idtpoperador)
    {
        $query = <<<SQL
            INSERT INTO OPERADOR
                        (CDOPERADOR, IDOPERATIVO, IDATIVO, NMOPERADOR, NMACESSO, DSEMAILOPER, CDPAIS, CDSENHAOPERWEB, NRPARCNEGOCIO, IDOPERMASTER,
                        CDCLIENTE, NRORG, NRORGTRAB, IDOPERACESSO, NRORGINCLUSAO,
                        CDOPERINCLUSAO, DTINCLUSAO, DTULTATUOPER, NRESTRUTTRAB, IDTPOPERADOR)
                 VALUES (:CDOPERADOR, :IDOPERATIVO, :IDATIVO, :NMOPERADOR, :NMACESSO, :DSEMAILOPER, :CDPAIS, :CDSENHAOPERWEB,
                         :NRPARCNEGOCIO, :IDOPERMASTER, :CDCLIENTE, :NRORG, :NRORGTRAB, :IDOPERACESSO,
                         :NRORGINCLUSAO, :CDOPERINCLUSAO, SYSDATE, SYSDATE, :NRESTRUTTRAB, :IDTPOPERADOR)
SQL;

        $params = array (
            "CDOPERADOR"          => $operatorCode,
            "IDOPERATIVO"         => $activeOperatorId,
            "IDATIVO"             => $activeId,
            "NMOPERADOR"          => $operatorName,
            "NMACESSO"            => $accessName,
            "DSEMAILOPER"         => $operatorEmail,
            "CDPAIS"              => $countryCode,
            "NRPARCNEGOCIO"       => $businessPartnership,
            "CDCLIENTE"           => $clientCode,
            "NRORG"               => $organizationId,
            "NRORGTRAB"           => $organizationId,
            "IDOPERACESSO"        => $accessOperatorId,
            "NRORGINCLUSAO"       => $organizationInclusion,
            "CDOPERINCLUSAO"      => $operatorInclusionCode,
            "CDSENHAOPERWEB"      => $password,
            "IDOPERMASTER"        => $idopermaster,
            "NRESTRUTTRAB"        => $nrestruttrab,
            "IDTPOPERADOR"        => $idtpoperador
        );

        try {
            InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };

    }

    public function saveOperatorWithCdFilial(
            $operatorCode, $activeOperatorId, $activeId, $operatorName, $accessName, $operatorEmail,
            $countryCode, $businessPartnership, $clientCode, $organizationId, $accessOperatorId,
            $selectedProfile, $organizationInclusion, $operatorInclusionCode, $password, $idopermaster, $nrestruttrab,
            $idtpoperador, $cdfilial)
    {
        $query = <<<SQL
            INSERT INTO OPERADOR
                        (CDOPERADOR, IDOPERATIVO, IDATIVO, NMOPERADOR, NMACESSO, DSEMAILOPER, CDPAIS, CDSENHAOPERWEB, NRPARCNEGOCIO, IDOPERMASTER,
                        CDCLIENTE, NRORG, NRORGTRAB, IDOPERACESSO, CDPERFILSELECIONADO, NRORGINCLUSAO,
                        CDOPERINCLUSAO, DTINCLUSAO, DTULTATUOPER, NRESTRUTTRAB, IDTPOPERADOR, CDFILIALSELECIONADA)
                 VALUES (:CDOPERADOR, :IDOPERATIVO, :IDATIVO, :NMOPERADOR, :NMACESSO, :DSEMAILOPER, :CDPAIS, :CDSENHAOPERWEB,
                         :NRPARCNEGOCIO, :IDOPERMASTER, :CDCLIENTE, :NRORG, :NRORGTRAB, :IDOPERACESSO, :CDPERFILSELECIONADO,
                         :NRORGINCLUSAO, :CDOPERINCLUSAO, SYSDATE, SYSDATE, :NRESTRUTTRAB, :IDTPOPERADOR, :CDFILIALSELECIONADA)
SQL;

        $params = array (
            "CDOPERADOR"          => $operatorCode,
            "IDOPERATIVO"         => $activeOperatorId,
            "IDATIVO"             => $activeId,
            "NMOPERADOR"          => $operatorName,
            "NMACESSO"            => $accessName,
            "DSEMAILOPER"         => $operatorEmail,
            "CDPAIS"              => $countryCode,
            "NRPARCNEGOCIO"       => $businessPartnership,
            "CDCLIENTE"           => $clientCode,
            "NRORG"               => $organizationId,
            "NRORGTRAB"           => $organizationId,
            "IDOPERACESSO"        => $accessOperatorId,
            "CDPERFILSELECIONADO" => $selectedProfile,
            "NRORGINCLUSAO"       => $organizationInclusion,
            "CDOPERINCLUSAO"      => $operatorInclusionCode,
            "CDSENHAOPERWEB"      => $password,
            "IDOPERMASTER"        => $idopermaster,
            "NRESTRUTTRAB"        => $nrestruttrab,
            "IDTPOPERADOR"        => $idtpoperador,
            "CDFILIALSELECIONADA" => $cdfilial
        );

        try {
            InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function editOperator(
        $operatorCode, $activeOperatorId, $activeId, $operatorName, $accessName, $operatorEmail,
        $countryCode, $clientCode, $organizationId, $workOrganizationId, $userId, $businessPartnership, $idopermaster, $nrestruttrab, $idtpoperador)
    {
        $query = "UPDATE OPERADOR
                     SET IDOPERATIVO     = :IDOPERATIVO,
                         IDATIVO         = :IDATIVO,
                         NMOPERADOR      = :NMOPERADOR,
                         NMACESSO        = :NMACESSO,
                         DSEMAILOPER     = :DSEMAILOPER,
                         CDPAIS          = :CDPAIS,
                         CDCLIENTE       = :CDCLIENTE,
                         CDOPERULTATU    = :CDOPERULTATU,
                         NRORGULTATU     = :NRORG,
                         DTULTATUOPER    = SYSDATE,
                         NRPARCNEGOCIO   = :NRPARCNEGOCIO,
                         IDOPERMASTER    = :IDOPERMASTER,
                         NRESTRUTTRAB    = :NRESTRUTTRAB,
                         IDTPOPERADOR    = :IDTPOPERADOR
                   WHERE CDOPERADOR      = :CDOPERADOR
                     AND NRORG           = :NRORG";

        $params = array (
            "IDOPERATIVO"   => $activeOperatorId,
            "IDATIVO"       => $activeId,
            "NMOPERADOR"    => $operatorName,
            "NMACESSO"      => $accessName,
            "DSEMAILOPER"   => $operatorEmail,
            "CDPAIS"        => $countryCode,
            "CDCLIENTE"     => $clientCode,
            "NRORG"         => $organizationId,
            "CDOPERULTATU"  => $userId,
            "CDOPERADOR"    => $operatorCode,
            "NRPARCNEGOCIO" => $businessPartnership,
            "IDOPERMASTER"  => $idopermaster,
            "NRESTRUTTRAB"  => $nrestruttrab,
            "IDTPOPERADOR"  => $idtpoperador
        );

        try {
            InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function deleteRelatedBranchAndProfile($operatorCode, $organizationId)
    {
        $query = "UPDATE OPERADOR
                     SET CDPERFILSELECIONADO     =  null,
                         CDFILIALSELECIONADA         = null
                   WHERE CDOPERADOR      = :CDOPERADOR
                     AND NRORG           = :NRORG";

        $params = array (
            "NRORG"         => $organizationId,
            "CDOPERADOR"    => $operatorCode
        );

        try {
            InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }


    public function getOperatorData($operatorCode, $organizationId){
     $query = "SELECT CDOPERADOR, CDPERFILSELECIONADO, CDFILIALSELECIONADA, NMOPERADOR FROM OPERADOR
                   WHERE NRORG = :NRORG 
                     AND CDOPERADOR = :CDOPERADOR";

        $params = array(
            "NRORG"         => $organizationId,
            "CDOPERADOR"    => $operatorCode
        );

        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
            return $std->fetchAll()[0];
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function deleteOperator($operatorCode, $organizationId)
    {
        $query = "DELETE 
                    FROM OPERADOR 
                   WHERE CDOPERADOR = :CDOPERADOR
                     AND NRORG      = :NRORG";

        $params = array(
            "CDOPERADOR"    => $operatorCode,
            "NRORG"         => $organizationId
        );

        try {
            InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function findAccessProfileByOperator($organizationId, $businessPartnership)
    {
        $query = "SELECT NRPERFILACESSO, NMPERFILACESSO, IDPERFILACESSOADM 
                    FROM PERFILACESSO 
                   WHERE NRORG = :NRORG 
                     AND IDATIVO != 'N' 
                     AND NRPERFILACESSO NOT IN (SELECT NRPERFILACESSO FROM PERFACESSPARC WHERE :NRPARCNEGOCIO = NRPARCNEGOCIO)
                ORDER BY NMPERFILACESSO ASC";

        $params = array(
            "NRORG"         => $organizationId,
            "NRPARCNEGOCIO" => $businessPartnership
        );

        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
            return $std->fetchAll();
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function findAccessProfileByControlProfile($organizationId, $operador, $operadorcont, $cdfilial)
    {
        $queryProfileControl = "SELECT PFC.NRPERFACESSCONT, PFAC.NMPERFILACESSO AS NMPERFILACESSOCONT, PFC.NRPERFILACESSO, PFA.NMPERFILACESSO AS NMPERFILACESSO, 
                        :CDFILIAL AS CDFILIAL, :CDOPERADORCONT AS CDOPERADORCONT, :CDOPERADOR AS CDOPERADOR, PFF.CDFILIAL
                    FROM PERFACESSCONTROL PFC 
                        JOIN PERFILACESSO PFAC
                            ON PFC.NRPERFACESSCONT = PFAC.NRPERFILACESSO
                        JOIN PERFILACESSO PFA 
                            ON PFC.NRPERFILACESSO = PFA.NRPERFILACESSO
                        JOIN PERFACESSFILI PFF 
                            ON PFF.CDFILIAL = :CDFILIAL
                            AND PFF.CDOPERADOR = :CDOPERADORCONT
                    WHERE PFC.NRORG = :NRORG 
                        AND PFF.NRPERFILACESSO = (SELECT CDPERFILSELECIONADO FROM OPERADOR WHERE CDOPERADOR = :CDOPERADORCONT)
                        AND PFC.NRPERFACESSCONT = (SELECT CDPERFILSELECIONADO FROM OPERADOR WHERE CDOPERADOR = :CDOPERADORCONT)
                        AND PFA.NRPERFILACESSO NOT IN (SELECT NRPERFILACESSO FROM PERFACESSFILI WHERE CDFILIAL = :CDFILIAL AND CDOPERADOR = :CDOPERADOR)                        
                         ORDER BY PFC.NRPERFACESSCONT, PFC.NRPERFILACESSO";

        $queryGeral = "SELECT * FROM PERFILACESSO
                        WHERE NRORG = :NRORG
                        AND NRPERFILACESSO NOT IN (SELECT NRPERFILACESSO FROM PERFACESSFILI WHERE CDFILIAL = :CDFILIAL AND CDOPERADOR = :CDOPERADOR)
                         ORDER BY NRPERFILACESSO";

        $queryCheck = "SELECT COUNT(*) AS QTD FROM PERFACESSCONTROL";

        $params = array(
            "NRORG"         => $organizationId,
            "CDOPERADOR" => $operador,
            "CDOPERADORCONT" => $operadorcont,
            "CDFILIAL" => $cdfilial
        );

        $paramsGeral = array(
            "NRORG"         => $organizationId,
            "CDOPERADOR" => $operador,
            "CDFILIAL" => $cdfilial
        );
        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($queryCheck);
            $std->execute();
            $checkControlProfile = $std->fetchAll();
            $query = $checkControlProfile[0]['QTD'] > 0 ? $queryProfileControl : $queryGeral;
            $params = $checkControlProfile[0]['QTD'] > 0 ? $params : $paramsGeral;
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
            return $std->fetchAll();
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function findIdAdmAccessProfile($organizationId, $accessProfiles)
    {
        $query = "SELECT IDPERFILACESSOADM
                    FROM PERFILACESSO 
                   WHERE NRORG = :NRORG 
                     AND IDATIVO != 'N' 
                     AND NRPERFILACESSO IN ($accessProfiles)";

        $params = array(
            "NRORG"         => $organizationId
        );

        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
            return $std->fetchAll();
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }


    public function createBusinessPartnership($businessPartnership, $organizationId, $operatorName, $cpf, $activeId, $operatorInclusionCode, $inscriptionTypeCode)
    {
        $query = "INSERT INTO PARCNEGOCIO
                              (NRPARCNEGOCIO, NRORG, NMPRINCIPALPARC, NMSECUNDARIPARC, NRINSCRICAOPARC, IDATIVO,
                              DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO, CDTIPOPARCPRINCIPAL, CDTIPOINSCRICAO,
                              IDPESSOAFISICA, IDINSTITUICAO, IDPARCFUNDIDO)
                      VALUES (:NRPARCNEGOCIO, :NRORG, :NMOPERADOR, :NMOPERADOR, :NRINSCRICAOPARC, :IDATIVO,
                                SYSDATE, :NRORG, :CDOPERINCLUSAO, :CDTIPOPARCPRINCIPAL, :CDTIPOINSCRICAO,
                              :IDPESSOAFISICA, :IDINSTITUICAO, :IDPARCFUNDIDO)";

        $params = array (
            "NRPARCNEGOCIO"       => $businessPartnership,
            "NRORG"               => $organizationId,
            "NMOPERADOR"          => $operatorName,
            "NRINSCRICAOPARC"     => $cpf,
            "IDATIVO"             => 'S',
            "CDOPERINCLUSAO"      => $operatorInclusionCode,
            "CDTIPOPARCPRINCIPAL" => 'OPERADOR',
            "CDTIPOINSCRICAO"     => $inscriptionTypeCode,
            "IDPESSOAFISICA"      => 'S',
            "IDINSTITUICAO"       => 'N',
            "IDPARCFUNDIDO"       => 'N'
        );

        try {
            InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function updateBusinessPartnershipOperator($businessPartnership, $operatorCode, $organizationId) {
        $query = "UPDATE OPERADOR
                     SET NRPARCNEGOCIO = :NRPARCNEGOCIO
                   WHERE CDOPERADOR = :CDOPERADOR
                     AND NRORG = :NRORG";

        $params = array (
            "NRPARCNEGOCIO" => $businessPartnership,
            "NRORG"         => $organizationId,
            "CDOPERADOR"    => $operatorCode
        );

        try {
            InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function createPartnership($businessPartnership, $organizationId, $operatorInclusionCode, $partnerTipeCode, $partnerNumber, $activeId)
    {
        $query = "INSERT INTO PARCERIA
                               (NRPARCERIA, NRPARCNEGOCIO, CDTIPOPARCEIRO, NRORG, IDATIVO,
                               DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO)
                       VALUES (:NRPARCERIA, :NRPARCNEGOCIO, :CDTIPOPARCEIRO, :NRORG, :IDATIVO,
                               SYSDATE, :NRORG, :CDOPERINCLUSAO)";

        $params = array(
            "NRPARCERIA"     => $partnerNumber,
            "NRPARCNEGOCIO"  => $businessPartnership,
            "NRORG"          => $organizationId,
            "CDTIPOPARCEIRO" => $partnerTipeCode,
            "IDATIVO"        => $activeId,
            "CDOPERINCLUSAO" => $operatorInclusionCode
        );

        try {
            InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function createRelationshipOragnizationOperator($organizationOperator, $workOrganizationId,
        $operatorCode, $organizationId, $operatorInclusionCode, $activeId)
    {
        $query = "INSERT INTO ORGANIZACAOOPER
                              (NRORGANIZACAOOPER, NRORGREL, CDOPERADOR, DTINCLUSAO, NRORGINCLUSAO,
                               CDOPERINCLUSAO, NRORG, IDATIVO)
                       VALUES (:NRORGANIZACAOOPER, :NRORGTRAB, :CDOPERADOR, SYSDATE, :NRORG,
                               :CDOPERINCLUSAO, :NRORG, :IDATIVO)";

        $params = array (
            "NRORGANIZACAOOPER" => $organizationOperator,
            "NRORGTRAB"         => $workOrganizationId,
            "CDOPERADOR"        => $operatorCode,
            "CDOPERINCLUSAO"    => $operatorInclusionCode,
            "NRORG"             => $organizationId,
            "IDATIVO"           => $activeId
        );

        try {
            InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function checkIfOperatorCode($operatorCode, $organizationId)
    {
        $query = "SELECT COUNT(*) AS COUNT
                    FROM OPERADOR
                   WHERE CDOPERADOR = :CDOPERADOR
                     AND NRORG = :NRORG";

        $params = array(
            'CDOPERADOR' => $operatorCode,
            'NRORG'      => $organizationId
        );

        try {
            InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function findOrganization($organizationId)
    {
        $query = "SELECT NRORG, NMORGANIZACAO
                    FROM ORGANIZACAO
                   WHERE NRORG = :NRORG";

        $params = array(
            "NRORG" => $organizationId
        );

        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
            return $std->fetchAll();
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }

    public function findOrganizationZhuLog()
    {
        $query = "SELECT NRORG AS ID, NMORGANIZACAO as NAME
                    FROM ORGANIZACAO
                   WHERE IDATIVO = 'S'";

        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute();
            return $std->fetchAll();
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }

    public function findPartnerAccessProfile($filters, $organizationId, $order = '')
    {
        $query = "SELECT CDOPERULTATU, NRORGULTATU, CDOPERINCLUSAO, NRORGINCLUSAO, DTULTATU, DTINCLUSAO, IDATIVO, NRORG,
                         DTVALPERFILPARC, NRPERFILACESSO, NRPARCNEGOCIO, NRPERFACESSPARC, NMPERFILACESSO, TO_CHAR(DTVALIDADE, 'DD/MM/YYYY') AS DTVALIDADE
                    FROM (SELECT PEPA.CDOPERULTATU,  PEPA.NRORGULTATU,  PEPA.CDOPERINCLUSAO,  PEPA.NRORGINCLUSAO,  PEPA.DTULTATU,
                                 PEPA.DTINCLUSAO,  PEPA.IDATIVO, PEPA.NRORG,  PEPA.DTVALPERFILPARC,  PEPA.NRPERFILACESSO,
                                 PEPA.NRPARCNEGOCIO,  PEPA.NRPERFACESSPARC,  PERF.NMPERFILACESSO, PEPA.DTVALIDADE
                            FROM PERFACESSPARC PEPA
                            JOIN PERFILACESSO PERF ON PERF.NRPERFILACESSO = PEPA.NRPERFILACESSO AND PERF.NRORG = PEPA.NRORG)
                   WHERE NRORG = :NRORG
                $filters
                $order";
        $params = array(
            'NRORG'   => $organizationId
        );
        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
            return $std->fetchAll();
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function findBranchAccessProfile($filters, $organizationId, $order = '')
    {
        $query = "SELECT CDOPERULTATU, DTVALIDADE, NRORGULTATU, CDOPERINCLUSAO, NRORGINCLUSAO, DTULTATU, DTINCLUSAO, IDATIVO, NRORG,
                          NRPERFILACESSO, NMPERFILACESSO, CDFILIAL, CDOPERADOR, NMFILIAL,
                           NMPAIS, NMESTADO, NMMUNICIPIO, NMREGIAO
                    FROM (SELECT DISTINCT PEFI.CDOPERULTATU,  PEFI.NRORGULTATU,  PEFI.CDOPERINCLUSAO,  PEFI.NRORGINCLUSAO,  PEFI.DTULTATU,
                                 PEFI.DTINCLUSAO,  PEFI.IDATIVO, PEFI.NRORG, PEFI.CDFILIAL, PEFI.CDOPERADOR, PEFI.NRPERFILACESSO, 
                                 PERF.NMPERFILACESSO, PEFI.DTVALIDADE, FI.NMFILIAL,
                                  PA.NMPAIS, ES.NMESTADO, MU.NMMUNICIPIO, RE.NMREGIAO
                            FROM PERFACESSFILI PEFI
                            JOIN FILIAL FI ON FI.CDFILIAL = PEFI.CDFILIAL AND FI.NRORG = PEFI.NRORG
                            JOIN PERFILACESSO PERF ON PERF.NRPERFILACESSO = PEFI.NRPERFILACESSO AND PERF.NRORG = PEFI.NRORG
                            JOIN ENDEFILI EM 
                                    ON FI.CDFILIAL = EM.CDFILIAL
                                    AND FI.NRORG = EM.NRORG
                                    AND EM.IDTPENDEFILI = 'P'
                            LEFT JOIN PAIS PA 
                                    ON EM.CDPAIS = PA.CDPAIS
                                    AND EM.NRORG = PA.NRORG
                            LEFT JOIN ESTADO ES
                                    ON EM.SGESTADO = ES.SGESTADO
                                    AND EM.CDPAIS = ES.CDPAIS
                                    AND EM.NRORG = ES.NRORG
                            LEFT JOIN MUNICIPIO MU 
                                    ON EM.CDMUNICIPIO = MU.CDMUNICIPIO
                                    AND EM.CDPAIS = MU.CDPAIS
                                    AND EM.SGESTADO = MU.SGESTADO
                                    AND EM.NRORG = MU.NRORG
                            LEFT JOIN REGIAO RE 
                                    ON FI.CDREGIAO = RE.CDREGIAO
                                    AND EM.CDPAIS = RE.CDPAIS
                                    AND EM.SGESTADO = RE.SGESTADO
                                    AND EM.NRORG = RE.NRORG)
                   WHERE NRORG = :NRORG
                $filters
                $order";
        $params = array(
            'NRORG'   => $organizationId
        );
        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
            return $std->fetchAll();
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function deletePartnerAccessProfile($partnerAccessProfileNumber, $organizationId)
    {
        $query = "DELETE 
                    FROM PERFACESSPARC
                   WHERE NRPERFACESSPARC = :NRPERFACESSPARC
                     AND NRORG           = :NRORG";

        $params = array(
            'NRPERFACESSPARC'   => $partnerAccessProfileNumber,
            'NRORG'             => $organizationId
        );

        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function deleteBranchAccessProfile($branchAccessProfileNumber, $operator, $branch, $organizationId)
    {
        $query = "DELETE 
                    FROM PERFACESSFILI
                   WHERE NRPERFILACESSO = :NRPERFILACESSO
                     AND CDOPERADOR = :CDOPERADOR
                     AND CDFILIAL = :CDFILIAL
                     AND NRORG           = :NRORG";

        $params = array(
            'NRPERFILACESSO'   => $branchAccessProfileNumber,
            'CDOPERADOR'        => $operator,
            'CDFILIAL'          => $branch,
            'NRORG'             => $organizationId
        );

        try {
            InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function saveAccessProfileByOperator($accessProfile)
    {
        $query = "INSERT INTO PERFACESSPARC
                              (NRPERFACESSPARC, NRORG, NRPARCNEGOCIO, NRPERFILACESSO, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO, DTVALIDADE)
                       VALUES (:NRPERFACESSPARC, :NRORG, :NRPARCNEGOCIO, :NRPERFILACESSO, SYSDATE, :NRORG, :CDOPERINCLUSAO, TRUNC(TO_DATE(:DTVALIDADE, 'DD/MM/YYYY')))";

        $params = array(
            "NRPERFACESSPARC"   => $accessProfile['NRPERFACESSPARC'],
            "NRORG"             => $accessProfile['NRORG'],
            "NRPARCNEGOCIO"     => $accessProfile['NRPARCNEGOCIO'],
            "NRPERFILACESSO"    => $accessProfile['NRPERFILACESSO'],
            "CDOPERINCLUSAO"    => $accessProfile['CDOPERINCLUSAO'],
            "DTVALIDADE"        => $accessProfile['DTVALIDADE'],
        );

        try {
            InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function saveAccessProfileByControlProfile($accessProfile)
    {
        $query = "INSERT INTO PERFACESSFILI
                              (CDFILIAL, NRORG, CDOPERADOR, NRPERFILACESSO, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO, DTVALIDADE)
                       VALUES (:CDFILIAL, :NRORG, :CDOPERADOR, :NRPERFILACESSO, SYSDATE, :NRORG, :CDOPERINCLUSAO, TRUNC(TO_DATE(:DTVALIDADE, 'DD/MM/YYYY')))";

        $params = array(
            "CDFILIAL"          => $accessProfile['CDFILIAL'],
            "NRORG"             => $accessProfile['NRORG'],
            "NRPERFILACESSO"    => $accessProfile['NRPERFILACESSO'],
            "CDOPERINCLUSAO"    => $accessProfile['CDOPERINCLUSAO'],
            "CDOPERADOR"        => $accessProfile['CDOPERADOR'],
            "DTVALIDADE"        => $accessProfile['DTVALIDADE']
        );

        try {
            InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function editAccessProfileByOperator($partnerAccessProfileNumber, $activeId, $organizationId, $dtValidade)
    {
        $query = "UPDATE PERFACESSPARC
                     SET IDATIVO         = :IDATIVO,
                         DTVALIDADE      = TRUNC(TO_DATE(:DTVALIDADE, 'DD/MM/YYYY'))
                   WHERE NRPERFACESSPARC = :NRPERFACESSPARC
                     AND NRORG           = :NRORG";

        $params = array(
            "IDATIVO"           => $activeId,
            "NRPERFACESSPARC"   => $partnerAccessProfileNumber,
            "NRORG"             => $organizationId,
            "DTVALIDADE"        => $dtValidade
        );

        try {
            InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function editAccessProfileByControlProfile($accessProfileNumber,  $operador, $unidade, $activeId, $organizationId, $dtValidade)
    {
        $query = "UPDATE PERFACESSFILI
                     SET IDATIVO         = :IDATIVO,
                         DTVALIDADE      = TRUNC(TO_DATE(:DTVALIDADE, 'DD/MM/YYYY'))
                   WHERE NRPERFILACESSO = :NRPERFILACESSO
                     AND CDOPERADOR = :CDOPERADOR
                     AND CDFILIAL = :CDFILIAL
                     AND NRORG           = :NRORG";

        $params = array(
            "IDATIVO"           => $activeId,
            "NRPERFILACESSO"    => $accessProfileNumber,
            "NRORG"             => $organizationId,
            "CDOPERADOR"        => $operador,
            "CDFILIAL"          => $unidade,
            "DTVALIDADE"        => $dtValidade
        );

        try {
            InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function updateBusinessPartner($cpf, $inscriptionTypeCode, $operatorName, $organizationId, $userId, $businessPartnership)
    {
        $query = "UPDATE PARCNEGOCIO
                      SET NRINSCRICAOPARC = :NRINSCRICAOPARC,
                          CDTIPOINSCRICAO = :CDTIPOINSCRICAO,
                          NMPRINCIPALPARC = :NMOPERADOR,
                          NMSECUNDARIPARC = :NMOPERADOR,
                          DTULTATU        =  SYSDATE,
                          NRORGULTATU     = :NRORG,
                          CDOPERULTATU    = :NRCDOPERADOR
                    WHERE NRPARCNEGOCIO   = :NRPARCNEGOCIO
                      AND NRORG           = :NRORG";

        $params = array(
            "NRPARCNEGOCIO"   => $businessPartnership,
            "NRORG"           => $organizationId,
            "NMOPERADOR"      => $operatorName,
            "NRINSCRICAOPARC" => $cpf,
            "NRCDOPERADOR"    => $userId,
            "CDTIPOINSCRICAO" => $inscriptionTypeCode,
        );

        try {
            InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
        }catch(\Exception $e){
            throw new \Exception($e->getMessage());
        };
    }

    public function inactiveAllPartnershipRelationships($userId, $organizationId, $businessPartnership, $activeId)
    {
        $query = "UPDATE RELACIONAPARC
                      SET IDATIVO         = :IDATIVO,
                          CDOPERULTATU    = :CDOPERULTATU,
                          NRORGULTATU     = :NRORGULTATU,
                          DTULTATU        = SYSDATE
                    WHERE NRPARCNEGRELAC  = :NRPARCNEGOCIO
                      AND NRORG           = :NRORG
                      AND NRTIPORELACIONA = :NRTIPORELACIONA";

        $params = array(
            "NRORG"           => $organizationId,
            "NRPARCNEGOCIO"   => $businessPartnership,
            "NRORGULTATU"     => $organizationId,
            "CDOPERULTATU"    => $userId,
            "IDATIVO"         => $activeId,
            "NRTIPORELACIONA" => 25
        );

        try{
            InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
        }
        catch(\Exception $e){
            throw new \Exception($e->getMessage());
        };
    }

    public function inactiveAllClientPartners($userId, $organizationId, $businessPartnership)
    {
        $query = "UPDATE INTECLIE
                     SET IDATIVO        = :IDATIVO,
                         CDOPERULTATU   = :CDOPERULTATU,
                         NRORGULTATU    = :NRORGULTATU,
                         DTULTATU       = SYSDATE
                   WHERE NRRELACIONAPARC IN (SELECT NRRELACIONAPARC
                                               FROM RELACIONAPARC
                                              WHERE NRPARCNEGRELAC  = :NRPARCNEGOCIO
                                                AND NRORG           = :NRORG
                                                AND NRTIPORELACIONA = :NRTIPORELACIONA)
                     AND NRORG = :NRORG";

        $params = array(
            "NRORG"           => $organizationId,
            "NRPARCNEGOCIO"   => $businessPartnership,
            "NRORGULTATU"     => $organizationId,
            "CDOPERULTATU"    => $userId,
            "IDATIVO"         => "N",
            "NRTIPORELACIONA" => 25
        );

        try{
            InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
        }catch(\Exception $e){
            throw new \Exception($e->getMessage());
        };
    }

    public function setActiveInactivePartnership($userId, $organizationId, $businessPartnership, $activeId)
    {
        $query = "UPDATE PARCERIA
                      SET IDATIVO        = :IDATIVO,
                          CDOPERULTATU   = :CDOPERULTATU,
                          NRORGULTATU    = :NRORGULTATU,
                          DTULTATU       = SYSDATE
                    WHERE NRPARCNEGOCIO  = :NRPARCNEGOCIO
                      AND NRORG          = :NRORG
                      AND CDTIPOPARCEIRO = :CDTIPOPARCEIRO";

        $params = array(
            "NRORG"          => $organizationId,
            "NRPARCNEGOCIO"  => $businessPartnership,
            "NRORGULTATU"    => $organizationId,
            "CDOPERULTATU"   => $userId,
            "IDATIVO"        => $activeId,
            "CDTIPOPARCEIRO" => "INTERLOCUTOR"
        );

        try{
            InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
        }catch(\Exception $e){
            throw new \Exception($e->getMessage());
        };
    }

    public function isThereInterlocutorPartnership($organizationId, $businessPartnership)
    {
        $query = "SELECT COUNT(*) AS COUNT
                     FROM PARCERIA
                    WHERE NRPARCNEGOCIO  = :NRPARCNEGOCIO
                      AND NRORG          = :NRORG
                      AND CDTIPOPARCEIRO = :CDTIPOPARCEIRO";

        $params = array(
            "NRORG"          => $organizationId,
            "NRPARCNEGOCIO"  => $businessPartnership,
            "CDTIPOPARCEIRO" => "INTERLOCUTOR"
        );

        try{
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
            $result = $std->fetchAll();
            return $result[0]["COUNT"];
        }catch(\Exception $e){
            throw new \Exception($e->getMessage());
        };
    }

    public function isTherePartnerRelationship($organizationId, $businessPartnership, $clientCode)
    {
        $query = "SELECT COUNT(*) AS COUNT
                     FROM RELACIONAPARC
                    WHERE NRPARCNEGRELAC  = :NRPARCNEGOCIO
                      AND NRORG           = :NRORG
                      AND NRTIPORELACIONA = :NRTIPORELACIONA
                      AND NRPARCNEGOCIO  IN (SELECT NRPARCNEGOCIO
                                               FROM CLIENTE
                                              WHERE CDCLIENTE = :CDCLIENTE
                                                AND NRORG     = :NRORG)";

        $params = array(
            "NRORG"           => $organizationId,
            "NRTIPORELACIONA" => 25,
            "NRPARCNEGOCIO"   => $businessPartnership,
            "CDCLIENTE"       => $clientCode
        );

        try{
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
            $result = $std->fetchAll();
            return $result[0]["COUNT"];
        }catch(\Exception $e){
            throw new \Exception($e->getMessage());
        };
    }

    public function activateClientPartner($organizationId, $businessPartnership, $clientCode, $userId, $activeId)
    {
        $query =  "UPDATE INTECLIE
                      SET IDATIVO         = :IDATIVO,
                          CDOPERULTATU    = :CDOPERULTATU,
                          NRORGULTATU     = :NRORGULTATU,
                          DTULTATU        = SYSDATE
                    WHERE NRRELACIONAPARC IN (SELECT NRRELACIONAPARC
                                                FROM RELACIONAPARC
                                               WHERE NRPARCNEGRELAC  = :NRPARCNEGOCIO
                                              AND NRORG           = :NRORG
                                              AND NRTIPORELACIONA = :NRTIPORELACIONA)
                      AND NRORG           = :NRORG
                      AND CDCLIENTE       = :CDCLIENTE";

        $params = array(
            "NRORG"           => $organizationId,
            "NRPARCNEGOCIO"   => $businessPartnership,
            "NRORGULTATU"     => $organizationId,
            "CDOPERULTATU"    => $userId,
            "IDATIVO"         => $activeId,
            "NRTIPORELACIONA" => 25,
            "CDCLIENTE"       => $clientCode
        );

        try{
            InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
        }catch(\Exception $e){
            throw new \Exception($e->getMessage());
        };
    }

    public function resetPassword($passwordWeb, $passwordDelphi, $cdOperador, $organizationId)
    {
        $query = "UPDATE OPERADOR
                     SET CDSENHAOPERWEB = :CDSENHAOPERWEB,
                         CDSENHOPER     = :CDSENHOPER,
                         IDSTATUSENHA   = 'A'
                   WHERE CDOPERADOR = :CDOPERADOR
                     AND NRORG       = :NRORG";

        $params = array(
            'CDSENHAOPERWEB' => $passwordWeb,
            'CDSENHOPER'     => $passwordDelphi,
            'CDOPERADOR'     => $cdOperador,
            'NRORG'          => $organizationId
        );

        try{
            InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
        }catch(\Exception $e){
            throw new \Exception($e->getMessage());
        };
    }

    public function updatePassword($passwordWeb, $passwordDelphi, $userId, $organizationId)
    {
        $query = "UPDATE OPERADOR
                     SET CDSENHAOPERWEB = :CDSENHAOPERWEB,
                         CDSENHOPER     = :CDSENHOPER,
                         IDSTATUSENHA   = 'A'
                   WHERE CDOPERADOR = :CDOPERADOR
                     AND NRORG      = :NRORG";

        $params = array(
            'CDSENHAOPERWEB' => $passwordWeb,
            'CDSENHOPER'     => $passwordDelphi,
            'CDOPERADOR'     => $userId,
            'NRORG'          => $organizationId
        );

        try {
            InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
        } catch(\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function uploadProfilePicture($imagePath, $userId, $organizationId)
    {
        $query = "UPDATE OPERADOR
                     SET DSIMAGEMOPER = :DSIMAGEMOPER
                   WHERE CDOPERADOR    = :CDOPERADOR
                     AND NRORG          = :NRORG";

        $params = array(
            'DSIMAGEMOPER'  => $imagePath,
            'CDOPERADOR'    => $userId,
            'NRORG'         => $organizationId
        );

        try{
            InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
        }catch(\Exception $e){
            throw new \Exception($e->getMessage());
        };
    }

    public function deleteRelatedBranch($cdfilial, $cdoperador, $organizationId)
    {
        $query = "DELETE 
                    FROM FILIOPER
                   WHERE CDFILIAL   = :CDFILIAL
                     AND CDOPERADOR = :CDOPERADOR
                     AND NRORG      = :NRORG";

        $params = array(
            'CDOPERADOR' => $cdoperador,
            'CDFILIAL'   => $cdfilial,
            'NRORG'      => $organizationId
        );

        try {
            InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function saveRelatedBranchByOperator($relatedBranch)
    {
        $query = "INSERT INTO FILIOPER
                              (NRORG, CDFILIAL, CDOPERADOR, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, NRORGFILIAL, IDATIVO, DTVALIDADE)
                       VALUES (:NRORG, :CDFILIAL, :CDOPERADOR, SYSDATE, :NRORG, :CDOPERINCLUSAO, :NRORG, :CDOPERINCLUSAO, :NRORG, 'S', TRUNC(TO_DATE(:DTVALIDADE, 'DD/MM/YYYY')))";

        $params = array(
            "CDFILIAL"          => $relatedBranch['CDFILIAL'],
            "NRORG"             => $relatedBranch['NRORG'],
            "CDOPERADOR"        => $relatedBranch['CDOPERADOR'],
            "CDOPERINCLUSAO"    => $relatedBranch['CDOPERINCLUSAO'],
            "DTVALIDADE"        => $relatedBranch['DTVALIDADE'],
        );

        try {
            InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function editRelatedBranchByOperator($cdfilial, $cdoperador, $activeId, $organizationId, $dtValidade)
    {
        $query = "UPDATE FILIOPER
                     SET IDATIVO         = :IDATIVO,
                         DTVALIDADE      = TRUNC(TO_DATE(:DTVALIDADE, 'DD/MM/YYYY'))
                   WHERE CDFILIAL        = :CDFILIAL
                     AND NRORG           = :NRORG
                     AND CDOPERADOR      = :CDOPERADOR";

        $params = array(
            "IDATIVO"           => $activeId,
            "CDFILIAL"          => $cdfilial,
            "CDOPERADOR"        => $cdoperador,
            "NRORG"             => $organizationId,
            "DTVALIDADE"        => $dtValidade
        );

        try {
            InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function deleteRelatedOrganization($nrorgrel, $cdoperador, $organizationId)
    {
        $query = "DELETE 
                    FROM ORGANIZACAOOPER
                   WHERE NRORGREL   = :NRORGREL
                     AND CDOPERADOR = :CDOPERADOR
                     AND NRORG      = :NRORG";

        $params = array(
            'CDOPERADOR' => $cdoperador,
            'NRORGREL'   => $nrorgrel,
            'NRORG'      => $organizationId
        );

        try {
            InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function saveRelatedOrganizationByOperator($relatedOrganization)
    {
        $query = "INSERT INTO ORGANIZACAOOPER
                              (NRORGANIZACAOOPER, NRORG, NRORGREL, CDOPERADOR, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, IDATIVO)
                       VALUES (:NRORGANIZACAOOPER, :NRORG, :NRORGREL, :CDOPERADOR, SYSDATE, :NRORG, :CDOPERINCLUSAO, :NRORG, :CDOPERINCLUSAO, 'S')";

        $params = array(
            "NRORGANIZACAOOPER" => $relatedOrganization['NRORGANIZACAOOPER'],
            "NRORGREL"          => $relatedOrganization['NRORGREL'],
            "NRORG"             => $relatedOrganization['NRORG'],
            "CDOPERADOR"        => $relatedOrganization['CDOPERADOR'],
            "CDOPERINCLUSAO"    => $relatedOrganization['CDOPERINCLUSAO']
        );

        try {
            InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function editRelatedOrganizationByOperator($nrorgrel, $cdoperador, $activeId, $organizationId)
    {
        $query = "UPDATE ORGANIZACAOOPER
                     SET IDATIVO         = :IDATIVO
                   WHERE NRORGREL        = :NRORGREL
                     AND CDOPERADOR      = :CDOPERADOR
                     AND NRORG           = :NRORG";

        $params = array(
            "IDATIVO"           => $activeId,
            "NRORGREL"          => $nrorgrel,
            "CDOPERADOR"        => $cdoperador,
            "NRORG"             => $organizationId
        );

        try {
            InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function importBranches($operatorCode, $organizationId, $branch, $operatorInclusionCode)
    {
        $query = "INSERT INTO FILIOPER
                              (CDOPERADOR, CDFILIAL, IDEXIBAVISO, DTULTATUFOPE, NRORG, DTINCLUSAO,
                              DTULTATU, NRORGINCLUSAO, NRORGULTATU, CDOPERULTATU, NRORGFILIAL,
                              IDATIVO)
                      VALUES (:CDOPERADOR, :CDFILIAL, 'S', SYSDATE, :NRORG, SYSDATE,
                              SYSDATE, :NRORG, :NRORG, :CDOPERINCLUSAO, :NRORG,
                              'S')";

        $params = array (
            "CDOPERADOR"          => $operatorCode,
            "NRORG"               => $organizationId,
            "CDOPERINCLUSAO"      => $operatorInclusionCode,
            "CDFILIAL"            => $branch
        );

        try {
            InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function getParamGeralByNrorg($nrOrg)
    {
        $query = "SELECT CDPICTPROD, CDPICTPRAT, IDUTCTRLACESWEB
                    FROM PARAMGERAL
                   WHERE NRORG = :NRORG";

        $params = array( 'NRORG' => $nrOrg );
        $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
        $std->execute($params);
        $result = $std->fetchAll();
        return !empty($result) ? $result[0] : $result;
    }

    public function findFilioper($filters, $order, $limit = 0, $offset = 0)
    {
        $query = <<<SQL
            SELECT DISTINCT CDOPERADOR, CDFILIAL, IDEXIBAVISO, NRORGFILIAL, DTULTATUFOPE, NRORG
                   DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, 
                   CDOPERULTATU, IDATIVO, NMFILIAL, TO_CHAR(DTVALIDADE, 'DD/MM/YYYY') AS DTVALIDADE, NRINSJURFILI,
                   NMPAIS, NMESTADO, NMMUNICIPIO, NMREGIAO
              FROM (SELECT CDOPERADOR, FIOP.CDFILIAL, IDEXIBAVISO, FIOP.NRORG AS NRORGFILIAL, DTULTATUFOPE,
                           FIOP.NRORG, FIOP.DTINCLUSAO, FIOP.DTULTATU, FIOP.NRORGINCLUSAO,
                           FIOP.CDOPERINCLUSAO, FIOP.NRORGULTATU, FIOP.CDOPERULTATU, FIOP.IDATIVO,
                           FILI.NMFILIAL, FIOP.DTVALIDADE, FILI.NRINSJURFILI,
                            PA.NMPAIS, ES.NMESTADO, MU.NMMUNICIPIO, RE.NMREGIAO
                      FROM FILIOPER FIOP
                      JOIN FILIAL FILI ON FILI.NRORG = FIOP.NRORG
                                      AND FILI.CDFILIAL = FIOP.CDFILIAL
                      JOIN ENDEFILI EM 
                            ON FILI.CDFILIAL = EM.CDFILIAL
                            AND FILI.NRORG = EM.NRORG
                            AND EM.IDTPENDEFILI = 'P'
                    LEFT JOIN PAIS PA 
                            ON EM.CDPAIS = PA.CDPAIS
                            AND EM.NRORG = PA.NRORG
                    LEFT JOIN ESTADO ES
                            ON EM.SGESTADO = ES.SGESTADO
                            AND EM.CDPAIS = ES.CDPAIS
                            AND EM.NRORG = ES.NRORG
                    LEFT JOIN MUNICIPIO MU 
                            ON EM.CDMUNICIPIO = MU.CDMUNICIPIO
                           AND EM.SGESTADO = MU.SGESTADO
                           AND EM.CDPAIS = MU.CDPAIS
                           AND EM.NRORG = MU.NRORG
                    LEFT JOIN REGIAO RE 
                            ON FILI.CDREGIAO = RE.CDREGIAO
                            AND EM.SGESTADO = RE.SGESTADO
                            AND EM.CDPAIS = RE.CDPAIS
                            AND EM.NRORG = RE.NRORG)
                                      
              WHERE 1 = 1
              $filters
              $order
SQL;
        $queryWithPagination = $this->getPaginatedQuery($query, $limit, $offset);
        return InstanceProvider::getEntityManager()->getConnection()->fetchAll($queryWithPagination);
    }

    public function findFilioperWithStatusBlock($filters, $order, $limit, $offset)
    {
        $query = <<<SQL
        SELECT DISTINCT
            CDOPERADOR,    CDFILIAL,    IDEXIBAVISO,    NRORGFILIAL,    DTULTATUFOPE,
            NRORG          DTINCLUSAO,    DTULTATU,    NRORGINCLUSAO,    CDOPERINCLUSAO,
            NRORGULTATU,    IDSTATUSBLOQ,   CDOPERULTATU,    IDATIVO,    NMFILIAL,
            TO_CHAR(DTVALIDADE, 'DD/MM/YYYY') AS DTVALIDADE, NMPAIS, NMESTADO, NMMUNICIPIO, NMREGIAO
        FROM
        (
        SELECT
            CDOPERADOR,    FIOP.CDFILIAL,    IDEXIBAVISO,   FIOP.NRORG AS NRORGFILIAL,    DTULTATUFOPE,
            FIOP.NRORG,    FIOP.DTINCLUSAO,  FIOP.DTULTATU, FIOP.NRORGINCLUSAO,  FIOP.CDOPERINCLUSAO,
            FIOP.NRORGULTATU,  FIOP.CDOPERULTATU,  FIOP.IDATIVO, FILI.NMFILIAL, FIOP.DTVALIDADE, FILI.IDSTATUSBLOQ,
            PA.NMPAIS, ES.NMESTADO, MU.NMMUNICIPIO, RE.NMREGIAO
        FROM
            FILIOPER FIOP
            JOIN FILIAL FILI 
                    ON FILI.NRORG = FIOP.NRORG
                    AND FILI.CDFILIAL = FIOP.CDFILIAL
            JOIN ENDEFILI EM 
                    ON FILI.CDFILIAL = EM.CDFILIAL
                    AND FILI.NRORG = EM.NRORG
                    AND EM.IDTPENDEFILI = 'P'
            LEFT JOIN PAIS PA 
                    ON EM.CDPAIS = PA.CDPAIS
                    AND EM.NRORG = PA.NRORG
            LEFT JOIN ESTADO ES
                    ON EM.SGESTADO = ES.SGESTADO
                    AND EM.CDPAIS = ES.CDPAIS
                    AND EM.NRORG = ES.NRORG
            LEFT JOIN MUNICIPIO MU 
                    ON EM.CDMUNICIPIO = MU.CDMUNICIPIO
                    AND EM.CDPAIS = MU.CDPAIS
                    AND EM.SGESTADO = MU.SGESTADO
                    AND EM.NRORG = MU.NRORG
            LEFT JOIN REGIAO RE 
                    ON FILI.CDREGIAO = RE.CDREGIAO
                    AND EM.CDPAIS = PA.CDPAIS
                    AND EM.SGESTADO = ES.SGESTADO
                    AND EM.NRORG = RE.NRORG)
              WHERE 1 = 1
              $filters
              $order
SQL;
        $queryWithPagination = $this->getPaginatedQuery($query, $limit, $offset);
        return InstanceProvider::getEntityManager()->getConnection()->fetchAll($queryWithPagination);
    }


    public function findUnidade($utlPerfilFilial, $operator, $operatorCont, $nrOrg, $utlBlock, $limit, $offset, $filters)
    {
        if($utlPerfilFilial != 'N'){
                $query = "SELECT * 
                            FROM (SELECT DISTINCT FILI.CDOPERADOR, FILI.CDFILIAL, FILI.NRORG,
                       FILI.DTINCLUSAO, FILI.DTULTATU, FILI.NRORGINCLUSAO, FILI.CDOPERINCLUSAO, FILI.NRORGULTATU,
                       FILI.CDOPERULTATU, FILI.IDATIVO, FILI.NMFILIAL, FILI.IDSTATUSBLOQ, FILI.NRINSJURFILI,
                        PA.NMPAIS, ES.NMESTADO, MU.NMMUNICIPIO, RE.NMREGIAO
                  FROM (SELECT PEFI.CDOPERADOR, FILI.CDFILIAL,
                               FILI.NRORG, FILI.DTINCLUSAO, FILI.DTULTATU, FILI.NRORGINCLUSAO,
                               FILI.CDOPERINCLUSAO, FILI.NRORGULTATU, FILI.CDOPERULTATU, FILI.IDATIVO,
                               FILI.NMFILIAL, PEFI.DTVALIDADE, FILI.IDSTATUSBLOQ, FILI.NRINSJURFILI, FILI.CDREGIAO
                          FROM PERFACESSFILI PEFI
                          JOIN FILIAL FILI ON FILI.NRORG = PEFI.NRORG
                                          AND FILI.CDFILIAL = PEFI.CDFILIAL) FILI
                        JOIN ENDEFILI EM 
                                ON FILI.CDFILIAL = EM.CDFILIAL
                                AND FILI.NRORG = EM.NRORG
                                AND EM.IDTPENDEFILI = 'P'
                        LEFT JOIN PAIS PA 
                                ON EM.CDPAIS = PA.CDPAIS
                                AND EM.NRORG = PA.NRORG
                        LEFT JOIN ESTADO ES
                               ON EM.CDPAIS = ES.CDPAIS
                           AND EM.SGESTADO = ES.SGESTADO
                           AND EM.NRORG = ES.NRORG
                        LEFT JOIN MUNICIPIO MU 
                              ON EM.CDPAIS = MU.CDPAIS
                              AND EM.SGESTADO = MU.SGESTADO
                              AND EM.CDMUNICIPIO = MU.CDMUNICIPIO
                              AND EM.NRORG = MU.NRORG
                        LEFT JOIN REGIAO RE 
                            ON FILI.CDREGIAO = RE.CDREGIAO
                           AND EM.CDPAIS = RE.CDPAIS
                           AND EM.SGESTADO = RE.SGESTADO
                           AND EM.NRORG = RE.NRORG
                        WHERE CDOPERADOR = :CDOPERADORCONT
                              AND EM.IDTPENDEFILI = 'P'
                            
                        AND FILI.CDFILIAL NOT IN (SELECT CDFILIAL FROM FILIOPER WHERE CDOPERADOR = :CDOPERADOR)
                            AND FILI.NRORG = :NRORG";
            if($utlBlock == 'S'){
                $query .= " AND FILI.IDSTATUSBLOQ = 'N'";
            }

            $query .= ") WHERE 1 = 1
                        $filters
                        ORDER BY CDFILIAL";
            $params = array(
                'CDOPERADOR' => $operator,
                'CDOPERADORCONT' => $operatorCont,
                'NRORG' => $nrOrg
                );
        } else {
            $query = "SELECT *
                        FROM (SELECT DISTINCT FILI.*, PA.NMPAIS, ES.NMESTADO, MU.NMMUNICIPIO, RE.NMREGIAO FROM 
                        FILIAL FILI 
                        JOIN ENDEFILI EM 
                                ON FILI.CDFILIAL = EM.CDFILIAL
                                AND FILI.NRORG = EM.NRORG
                                AND EM.IDTPENDEFILI = 'P'
                        LEFT JOIN PAIS PA 
                                ON EM.CDPAIS = PA.CDPAIS
                                AND EM.NRORG = PA.NRORG
                        LEFT JOIN ESTADO ES
                                ON EM.SGESTADO = ES.SGESTADO
                                AND EM.CDPAIS = ES.CDPAIS
                                AND EM.NRORG = ES.NRORG
                        LEFT JOIN MUNICIPIO MU 
                                ON EM.CDMUNICIPIO = MU.CDMUNICIPIO
                                AND EM.SGESTADO = MU.SGESTADO
                                AND EM.CDPAIS = MU.CDPAIS
                                AND EM.NRORG = MU.NRORG
                        LEFT JOIN REGIAO RE 
                                ON FILI.CDREGIAO = RE.CDREGIAO
                                AND EM.SGESTADO = RE.SGESTADO
                                AND EM.CDPAIS = RE.CDPAIS
                                AND EM.NRORG = RE.NRORG
                     WHERE FILI.CDFILIAL NOT IN (SELECT CDFILIAL FROM FILIOPER WHERE :CDOPERADOR = CDOPERADOR)
                        AND FILI.NRORG = :NRORG)
                        WHERE 1 = 1
                        $filters
                        ORDER BY CDFILIAL";
            $params = array(
                'CDOPERADOR' => $operator,
                'NRORG' => $nrOrg
                );
        }
        $queryWithPagination = $this->getPaginatedQuery($query, $limit, $offset);
        $std = InstanceProvider::getEntityManager()->getConnection()->prepare($queryWithPagination);
        $std->execute($params);
        $result = $std->fetchAll();
        return $result;
    }


    public function getAllBranches($filter, $filtersConditions, $limit, $offset){
        $query = <<<SQL
            SELECT *
            FROM (SELECT DISTINCT FILI.*,  PA.NMPAIS, ES.NMESTADO, MU.NMMUNICIPIO, RE.NMREGIAO
                    FROM FILIAL FILI
                        JOIN ENDEFILI EM 
                                ON FILI.CDFILIAL = EM.CDFILIAL
                                AND FILI.NRORG = EM.NRORG
                                AND EM.IDTPENDEFILI = 'P'
                        LEFT JOIN PAIS PA 
                                ON EM.CDPAIS = PA.CDPAIS
                                AND EM.NRORG = PA.NRORG
                        LEFT JOIN ESTADO ES
                               ON EM.CDPAIS = ES.CDPAIS
                           AND EM.SGESTADO = ES.SGESTADO
                           AND EM.NRORG = ES.NRORG
                        LEFT JOIN MUNICIPIO MU 
                              ON EM.CDPAIS = MU.CDPAIS
                              AND EM.SGESTADO = MU.SGESTADO
                              AND EM.CDMUNICIPIO = MU.CDMUNICIPIO
                              AND EM.NRORG = MU.NRORG
                        LEFT JOIN REGIAO RE 
                            ON FILI.CDREGIAO = RE.CDREGIAO
                           AND EM.CDPAIS = RE.CDPAIS
                           AND EM.SGESTADO = RE.SGESTADO
                           AND EM.NRORG = RE.NRORG
                    WHERE 1 = 1
                    $filter)
                    WHERE 1 = 1
                    $filtersConditions
                ORDER BY CDFILIAL
SQL;
        $queryWithPagination = $this->getPaginatedQuery($query, $limit, $offset);
        return InstanceProvider::getEntityManager()->getConnection()->fetchAll($queryWithPagination);

    }

    public function deleteFilioper($cdOperador, $cdFilial, $nrOrg)
    {
        $query = "DELETE
                    FROM FILIOPER
                   WHERE CDOPERADOR = :CDOPERADOR
                     AND CDFILIAL   = :CDFILIAL
                     AND NRORG      = :NRORG";

        $params = array(
            "CDOPERADOR" => $cdOperador,
            "CDFILIAL"   => $cdFilial,
            "NRORG"      => $nrOrg
        );

        try {
            InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function findParamGeral($organizationId)
    {

        $query = "SELECT IDGERACDOPERADOR, IDCPFREQUIRED
                    FROM PARAMGERAL
                   WHERE NRORG = :NRORG";

        $params = array(
            "NRORG" => $organizationId
        );

        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
            return $std->fetchAll();
        } catch (\Exception $e) {
            return array(array('IDGERACDOPERADOR' => "N"));
        };
    }

    public function findParamGeralValidationCpf($organizationId)
    {

        $query = "SELECT IDCPFREQUIRED
                    FROM PARAMGERAL
                   WHERE NRORG = :NRORG";

        $params = array(
            "NRORG" => $organizationId
        );

        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
            return $std->fetchAll();
        } catch (\Exception $e) {
            return array(array('IDCPFREQUIRED' => "N"));
        };
    }

    public function findIdUtlPerfilFilialParamGeral($organizationId)
    {
        $query = "SELECT IDUTLPERFILFILIAL
                    FROM PARAMGERAL
                   WHERE NRORG = :NRORG";

        $params = array(
            "NRORG" => $organizationId
        );

        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
            return $std->fetchAll();
        } catch (\Exception $e) {
            return array(array('IDUTLPERFILFILIAL' => "N"));
        };
    }

        public function findNrAccessprofileByOperator($organizationId, $operator)
    {
        $query = "SELECT CDPERFILSELECIONADO
                    FROM OPERADOR
                   WHERE NRORG = :NRORG
                    AND  CDOPERADOR = :CDOPERADOR";

        $params = array(
            "NRORG" => $organizationId,
            "CDOPERADOR" => $operator
        );

        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
            return $std->fetchAll();
        } catch (\Exception $e) {
            return array(array('CDPERFILSELECIONADO' => 'N'));
        };
    }

    public function findAccessProfileParams($organizationId, $nrAccessProfile)
    {
        $query = "SELECT IDBLOCKFILIAL
                    FROM PERFILACESSO
                   WHERE NRORG = :NRORG
                    AND  NRPERFILACESSO = :NRPERFILACESSO";

        $params = array(
            "NRORG" => $organizationId,
            "NRPERFILACESSO" => $nrAccessProfile
        );
        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
            return $std->fetchAll();
        } catch (\Exception $e) {
            return array(array('IDBLOCKFILIAL' => 'N'));
        };
    }

    public function getProfileCanBeEdit($operator, $operatorCont, $nrorg)
    {
        $query = "SELECT NRPERFACESSCONT FROM PERFACESSCONTROL
                    WHERE NRPERFACESSCONT IN (SELECT CDPERFILSELECIONADO FROM OPERADOR WHERE CDOPERADOR = :CDOPERADORCONT)
                    AND NRPERFILACESSO IN (SELECT NRPERFILACESSO FROM PERFACESSFILI WHERE CDOPERADOR = :CDOPERADOR )
                    AND NRORG = :NRORG";

        $params = array(
            "NRORG" => $nrorg,
            "CDOPERADOR" => $operator,
            "CDOPERADORCONT" => $operatorCont
        );

        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
            return $std->fetchAll();
        } catch (\Exception $e) {
            return array(array('IDBLOCKFILIAL' => 'N'));
        };
    }

    public function validarNovoCodigo($nrorg, $tabela, $campo)
    {
        $query = "SELECT $campo AS ID, NRSEQUENCIAL,
                         CASE WHEN NRSEQUENCIAL IS NULL
                              THEN '1'
                              WHEN $campo > TO_NUMBER(NRSEQUENCIAL)
                              THEN '2'
                              ELSE '3'
                          END AS VALIDACAO
                    FROM (SELECT MAX($campo) AS $campo
                            FROM (SELECT $campo
                                    FROM $tabela 
                                   WHERE REGEXP_LIKE($campo, '^[[:digit:]]+$')
                                     AND NRORG = :NRORG)),
                         (SELECT MAX(NRSEQUENCIAL) AS NRSEQUENCIAL
                            FROM NOVOCODIGO
                           WHERE CDCONTADOR = '$tabela'
                             AND NRORG = :NRORG)";

        $params = array(
            "NRORG" => $nrorg
        );

        return InstanceProvider::getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function alterNewCode($table, $newCode, $nrorg, $validation)
    {
        if($validation == '3') {
            //NewCode is correct!
            return;
        }

        if($validation == '1') {
            $query = "INSERT INTO NOVOCODIGO (CDCONTADOR, NRSEQUENCIAL, NRORG, DTINCLUSAO)
                           VALUES (:CDCONTADOR, :NEWCODE, :NRORG, SYSDATE)";
        } else {
            $query = "UPDATE NOVOCODIGO
                         SET NRSEQUENCIAL = :NEWCODE
                       WHERE CDCONTADOR = :CDCONTADOR
                         AND NRORG = :NRORG";
        }

        $params = array (
            "NEWCODE"    => $newCode,
            "CDCONTADOR" => $table,
            "NRORG"      => $nrorg
        );

        InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
    }

    public function findNrorgOperNotInserted($nrorg)
    {
        $query = "SELECT NVL(O.NRORGOPER, O.NRORG) AS NRORGOPER, O.CDOPERADOR
                    FROM OPERADOR O
                   WHERE NVL(O.NRORGOPER, O.NRORG) <> 1
                     AND O.NRORG = :NRORG
                     AND O.CDOPERADOR <> '#SUP_TEKNISA'
                     AND NOT EXISTS (SELECT R.ID 
                                       FROM ORGRELACOPER R
                                      WHERE R.NRORGOPER = NVL(O.NRORGOPER, O.NRORG)
                                        AND R.CDOPERADOR = O.CDOPERADOR
                                        AND R.NRORG = O.NRORG)";

        $params = array (
            "NRORG" => $nrorg
        );

        return InstanceProvider::getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function updateNrorgOper($nrorg, $nrorgoper, $cdoperador)
    {
        $query = "UPDATE OPERADOR
                     SET NRORGOPER = :NRORGOPER
                   WHERE NRORG = :NRORG
                     AND CDOPERADOR = :CDOPERADOR";

        $params = array (
            "NRORGOPER"  => $nrorgoper,
            "NRORG"      => $nrorg,
            "CDOPERADOR" => $cdoperador,
        );

        InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
    }

    public function fixNrorgOper($nrorg, $nrorgoper)
    {
        $query = "UPDATE OPERADOR
                     SET NRORGOPER = :NRORGOPER
                   WHERE NRORG = :NRORG
                     AND (NRORGOPER IS NULL OR NRORGOPER IN (0, 1))";

        $params = array (
            "NRORGOPER" => $nrorgoper,
            "NRORG"     => $nrorg
        );

        InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
    }

    public function insertOrgRelacOper($nrorgOper, $cdOperador, $nrorg)
    {
        $query = "INSERT INTO ORGRELACOPER (ID, NRORGOPER, CDOPERADOR, DTINCLUSAO, DTULTATU, NRORG)
                       VALUES (:ID, :NRORGOPER, :CDOPERADOR, SYSDATE, SYSDATE, :NRORG)";

        $params = array (
            "ID" => InstanceProvider::getNewCode()->getCodeByTableName('ORGRELACOPER', $nrorg, 1, 10, true),
            "NRORGOPER" => $nrorgOper,
            "CDOPERADOR" => $cdOperador,
            "NRORG" => $nrorg,
        );

        InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
    }

    public function deleteOrgRelacOper($cdOperador, $nrorg)
    {
        $query = "DELETE 
                    FROM ORGRELACOPER
                   WHERE CDOPERADOR = :CDOPERADOR
                     AND NRORG = :NRORG";

        $params = array (
            "CDOPERADOR" => $cdOperador,
            "NRORG" => $nrorg,
        );

        InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
    }

    public function findOperatorsByCode($operatorCode)
    {

        $query = "SELECT CDOPERADOR, NMOPERADOR, NRORG
                    FROM OPERADOR
                   WHERE CDOPERADOR = :CDOPERADOR";

        $params = array (
            "CDOPERADOR" => $operatorCode
        );

        $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
        $std->execute($params);
        return $std->fetchAll();
    }
    public function DeleteGrupoper($operator){
        $query  = "DELETE FROM OPERGRUPOP WHERE CDOPERADOR = :CDOPERADOR";
         $params = array (
            "CDOPERADOR" => $operator["CDOPERADOR"]
        );

        InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
    }

    public function InsertGrupoper($gropor,$operator,$nrperfilacesso){
        $query  = "INSERT INTO OPERGRUPOP
        (CDGRUPOPER, CDOPERADOR)
        SELECT
        DISTINCT CDGRUPOPER, :CDOPERADOR
        FROM GRUPOPER
        WHERE (INSTR(:CDGRUPOPER, CDGRUPOPER) > 0)
	";
	   $gropor = implode('_', $gropor);
         $params = array (

            "CDOPERADOR" => $operator["CDOPERADOR"],
            "CDGRUPOPER"=>$gropor
        );

        InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
    }

    public function updateActiveIdativoByIdoperativo($organizationId) {
        $query = "UPDATE OPERADOR
                     SET IDATIVO = 'S'
                   WHERE IDOPERATIVO = 'A'
                     AND IDATIVO = 'N'
                     AND NRORG = :NRORG";

        $params = array (
            "NRORG" => $organizationId,
        );

        try {
            InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

    public function updateInactiveIdativoByIdoperativo($organizationId) {
        $query = "UPDATE OPERADOR
                     SET IDATIVO = 'N'
                   WHERE IDOPERATIVO = 'I'
                     AND IDATIVO = 'S'
                     AND NRORG = :NRORG";

        $params = array (
            "NRORG" => $organizationId,
        );

        try {
            InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }
    
    public function validOperador($CDVENDEDOR, $CDOPERADOR)
    {
        $query = <<<SQL
        SELECT  CDOPERADOR 
        FROM 
          VENDEDOR V 
        WHERE V.CDVENDEDOR = $CDVENDEDOR
        AND CDOPERADOR <> $CDOPERADOR
SQL;
        return InstanceProvider::getEntityManager()->getConnection()->fetchAll($query);
    }
    
    public function updateVendedor($CDVENDEDOR, $CDOPERADOR) {
        
        $query = "
        UPDATE VENDEDOR
            SET     CDOPERADOR = :CDOPERADOR
            WHERE   CDVENDEDOR = :CDVENDEDOR";
            
        $params = array (
            "CDOPERADOR" => $CDOPERADOR,
            "CDVENDEDOR" => $CDVENDEDOR
        );
        
        try {
            InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }
    
    public function deleteVendedor($CDOPERADOR) {

        $query = "
        UPDATE VENDEDOR
            SET     CDOPERADOR = ''
            WHERE   CDOPERADOR = :CDOPERADOR";
        
        $params = array (
            "CDOPERADOR" => $CDOPERADOR
        );
        
        try {
            InstanceProvider::getEntityManager()->getConnection()->executeQuery($query, $params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

}
